@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
@@ -41,7 +41,7 @@ class MbtAPI {
   static bool GenerateTagger( int, char** );
   static bool GenerateTagger( const std::string& );
   static bool RunTagger( int, char** );
-  MbtAPI( const std::string& );
+  explicit MbtAPI( const std::string& );
   MbtAPI( const std::string&, TiCC::LogStream& );
   ~MbtAPI();
   bool isInit() const;
