@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
@@ -30,10 +30,12 @@
 
 #include "mbt/Pattern.h"
 #include "mbt/Sentence.h"
+#include "ticcutils/Timer.h"
 #include "timbl/TimblAPI.h"
 
 namespace TiCC {
   class LogStream;
+  class Timer;
 }
 
 namespace Tagger {
@@ -67,7 +69,7 @@ namespace Tagger {
 		    const Timbl::ValueDistribution * );
     void NextPath( StringHash&,
 		   const Timbl::TargetValue *,
-		   const Timbl::ValueDistribution*,
+		   const Timbl::ValueDistribution *,
 		   int );
     void ClearBest();
     void Shift( int, int );
@@ -156,6 +158,9 @@ namespace Tagger {
     std::vector<double> distance_array;
     std::vector<std::string> distribution_array;
     std::vector<double> confidence_array;
+    TiCC::Timer timer1;
+    TiCC::Timer timer2;
+    TiCC::Timer timer3;
 
     static void manifest();
     int makedataset( std::istream& infile, bool do_known );
@@ -195,6 +200,7 @@ namespace Tagger {
     std::string r_option_name;
     std::string L_option_name;
     std::string EosMark;
+    std::string Separators;
 
     PatTemplate Ktemplate;
     PatTemplate Utemplate;
