@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
@@ -237,7 +237,7 @@ struct more_second {
     int HartBeat = 0;
     size_t line_cnt = 0;
     sentence mySentence( Ktemplate, Utemplate );
-    while ( mySentence.read( infile, input_kind, EosMark, line_cnt ) ){
+    while ( mySentence.read( infile, input_kind, EosMark, Separators, line_cnt ) ){
       if ( mySentence.size() == 0 )
 	continue;
       // cerr << mySentence << endl;
@@ -260,7 +260,6 @@ struct more_second {
 	// of the words in the dictionary and the values
 	// of the features are stored in the testpattern
 	int swcn = 0;
-	int thisTagCode;
 	while( mySentence.nextpat( Action, TestPat,
 				   *kwordlist, TheLex,
 				   swcn ) ){
@@ -271,10 +270,14 @@ struct more_second {
 	    }
 	  }
 	  if ( !skip )
-	    for( int f=0; f < nslots; f++){
+	    for( int f=0; f < nslots; ++f ){
 	      outfile << indexlex( TestPat[f], TheLex ) << " ";
 	    }
-	  thisTagCode = TheLex.Hash( mySentence.gettag(swcn) );
+	  int thisTagCode = -1;
+#pragma omp critical (hasher)
+	  {
+	    thisTagCode = TheLex.Hash( mySentence.gettag(swcn) );
+	  }
 	  if ( !skip ){
 	    for ( auto const& it : mySentence.getEnrichments(swcn) ){
 	      outfile << it << " ";
@@ -405,7 +408,7 @@ struct more_second {
   //**** stuff to process commandline options *****************************
 
   const string mbt_create_short = "hV%:d:e:E:k:K:l:L:m:M:n:o:O:p:P:r:s:t:T:u:U:XD:";
-  const string mbt_create_long = "version";
+  const string mbt_create_long = "version,tabbed";
 
   bool TaggerClass::parse_create_args( TiCC::CL_Options& opts ){
     string value;
@@ -425,6 +428,9 @@ struct more_second {
       EosMark = value;
       cout << "  Sentence delimiter set to '" << EosMark << "'" << endl;
     }
+    if ( opts.extract( "tabbed" ) ){
+      Separators = "\t";
+    }
     if ( opts.extract( 'K', value ) ){
       K_option_name = value;
       knownoutfileflag = true; // there is a knownoutfile specified
@@ -551,6 +557,7 @@ struct more_second {
 	 << "\t-% <percentage> Filter Threshold for ambitag construction (default 5%)\n"
 	 << "\t-E <enriched tagged training corpus file> \n"
 	 << "\t-T <tagged training corpus file> \n"
+	 << "\t--tabbed use tabs as separator in TAGGED input. (default is all whitespace)\n"
 	 << "\t-O\"Timbl options\" (Note: NO SPACE between O and \"!!!)\n"
 	 << "\t   <options>   options to use for both Known and Unknown Words Case Base\n"
 	 << "\t   K: <options>   options to use for Known Words Case Base\n"
@@ -574,16 +581,15 @@ struct more_second {
   }
 
   int TaggerClass::CreateTagger( TiCC::CL_Options& opts ){
-    string value;
     if ( opts.is_present( 'h' ) ||
 	 opts.is_present( "help" ) ){
       gen_usage( "mbtg" );
-      return true;
+      return -1;
     }
     //
     // present yourself to the user
     //
-    cerr << "mbtg " << VERSION << " (c) CLST, ILK and CLiPS 1998 - 2016." << endl
+    cerr << "mbtg " << VERSION << " (c) CLST, ILK and CLiPS 1998 - 2018." << endl
 	 << "Memory Based Tagger Generator" << endl
 	 << "CLST  - Centre for Language and Speech Technology,"
 	 << "Radboud University" << endl
@@ -622,14 +628,26 @@ struct more_second {
   int TaggerClass::CreateTagger( const string& opt_string ){
     TiCC::CL_Options opts;
     opts.allow_args( mbt_create_short, mbt_create_long );
-    opts.parse_args( opt_string );
+    try {
+      opts.parse_args( opt_string );
+    }
+    catch ( exception& e ){
+      cerr << e.what() << endl;
+      return -1;
+    }
     return CreateTagger( opts );
   }
 
   int TaggerClass::CreateTagger( int argc, char* argv[] ){
     TiCC::CL_Options opts;
     opts.allow_args( mbt_create_short, mbt_create_long );
-    opts.parse_args( argc, argv );
+    try {
+      opts.parse_args( argc, argv );
+    }
+    catch ( exception& e ){
+      cerr << e.what() << endl;
+      return -1;
+    }
     return CreateTagger( opts );
   }
 
